from typing import List, Sequence, Iterable

import numpy as np

from allennlp import data as allen_data, data


@allen_data.BatchSampler.register("token_count")
class TokenCountBatchSampler(allen_data.BatchSampler):

    def __init__(self, word_batch_size: int = 2500, shuffle_dataset: bool = True):
        self._index = 0
        self._word_batch_size = word_batch_size
        self._shuffle = shuffle_dataset

    def get_batch_indices(self, instances: Sequence[data.Instance]) -> Iterable[List[int]]:
        dataset = list(instances)
        batches = []
        batch = []
        words_count = 0
        lengths = [len(instance.fields["sentence"].tokens) for instance in dataset]
        argsorted_lengths = np.argsort(lengths)
        for idx in argsorted_lengths:
            words_count += lengths[idx]
            batch.append(idx)
            if words_count > self._word_batch_size:
                batches.append(batch)
                words_count = 0
                batch = []

        if self._shuffle:
            indices = np.random.permutation(range(len(batches)))
            batches = np.array(batches)[indices].tolist()

        return batches

    def get_num_batches(self, instances: Sequence[data.Instance]) -> int:
        return sum(1 for _ in self.get_batch_indices(instances))
